import os
import shutil
from subprocess import check_output
import tarfile
import tempfile

import pytest

from rpmautospec import process_distgit


__here__ = os.path.dirname(__file__)


class TestProcessDistgit:
    """Test the rpmautospec.process_distgit module"""

    autorelease_autochangelog_cases = [
        (autorelease_case, autochangelog_case)
        for autorelease_case in ("unchanged", "with braces", "optional")
        for autochangelog_case in (
            "unchanged",
            "changelog case insensitive",
            "changelog trailing garbage",
            "line in between",
            "trailing line",
            "with braces",
            "missing",
            "optional",
        )
    ]

    @staticmethod
    def fuzz_spec_file(spec_file_path, autorelease_case, autochangelog_case):
        """Fuzz a spec file in ways which shouldn't change the outcome"""

        with open(spec_file_path, "r") as orig, open(spec_file_path + ".new", "w") as new:
            for line in orig:
                if line.startswith("Release:") and autorelease_case != "unchanged":
                    if autorelease_case == "with braces":
                        print("Release:        %{autorelease}", file=new)
                    elif autorelease_case == "optional":
                        print("Release:        %{?autorelease}", file=new)
                    else:
                        raise ValueError(f"Unknown autorelease_case: {autorelease_case}")
                elif line.strip() == "%changelog" and autochangelog_case != "unchanged":
                    if autochangelog_case == "changelog case insensitive":
                        print("%ChAnGeLoG", file=new)
                    elif autochangelog_case == "changelog trailing garbage":
                        print("%changelog with trailing garbage yes this works", file=new)
                    elif autochangelog_case == "line in between":
                        print("%changelog\n\n%autochangelog", file=new)
                        break
                    elif autochangelog_case == "trailing line":
                        print("%changelog\n%autochangelog\n", file=new)
                        break
                    elif autochangelog_case == "with braces":
                        print("%changelog\n%{autochangelog}", file=new)
                        break
                    elif autochangelog_case == "missing":
                        # do nothing, i.e. don't print a %changelog to file
                        break
                    elif autochangelog_case == "optional":
                        print("%changelog\n%{?autochangelog}", file=new)
                        break
                    else:
                        raise ValueError(f"Unknown autochangelog_case: {autochangelog_case}")
                else:
                    print(line, file=new, end="")

        os.rename(spec_file_path + ".new", spec_file_path)

    @pytest.mark.parametrize("autorelease_case,autochangelog_case", autorelease_autochangelog_cases)
    def test_process_distgit(self, autorelease_case, autochangelog_case):
        """Test the process_distgit() function"""
        with tempfile.TemporaryDirectory() as workdir:
            with tarfile.open(
                os.path.join(
                    __here__,
                    os.path.pardir,
                    "test-data",
                    "repodata",
                    "dummy-test-package-gloster-git.tar.gz",
                )
            ) as tar:
                tar.extractall(path=workdir)

            unpacked_repo_dir = os.path.join(workdir, "dummy-test-package-gloster")
            unprocessed_spec_file_path = os.path.join(
                unpacked_repo_dir,
                "dummy-test-package-gloster.spec",
            )

            if autorelease_case != "unchanged" or autochangelog_case != "unchanged":
                self.fuzz_spec_file(
                    unprocessed_spec_file_path, autorelease_case, autochangelog_case
                )

            process_distgit.process_distgit(unpacked_repo_dir)

            expected_spec_file_path = os.path.join(
                __here__,
                os.path.pardir,
                "test-data",
                "repodata",
                "dummy-test-package-gloster.spec.expected",
            )

            with tempfile.NamedTemporaryFile() as tmpspec:
                shutil.copy2(expected_spec_file_path, tmpspec.name)
                if autorelease_case != "unchanged" or autochangelog_case != "unchanged":
                    if autochangelog_case not in (
                        "changelog case insensitive",
                        "changelog trailing garbage",
                    ):
                        # "%changelog", "%ChAnGeLoG", ... stay verbatim, trick fuzz_spec_file() to
                        # leave the rest of the cases as is, the %autorelease macro is expanded.
                        autochangelog_case = "unchanged"
                    expected_spec_file_path = tmpspec.name
                    self.fuzz_spec_file(
                        expected_spec_file_path, autorelease_case, autochangelog_case
                    )

                rpm_cmd = ["rpm", "--define", "dist .fc32", "--specfile"]

                unprocessed_cmd = rpm_cmd + [unprocessed_spec_file_path]
                expected_cmd = rpm_cmd + [expected_spec_file_path]

                q_release = ["--qf", "%{release}\n"]
                assert check_output(unprocessed_cmd + q_release) == check_output(
                    expected_cmd + q_release
                )

                q_changelog = ["--changelog"]
                assert check_output(unprocessed_cmd + q_changelog) == check_output(
                    expected_cmd + q_changelog
                )
